#!/usr/bin/env python3.6m
"""
Renews vault token
"""
import time

import hvac


class VaultTokenRefresher(object):
    def __init__(
        self, vault_addr: str, token: str, sleep_time: int = 3600, forever: bool = False
    ):
        self.client = hvac.Client(url=vault_addr, token=token)
        self.sleep_time = sleep_time
        self.forever = forever

        self.renewal = None

    def info(self):
        lookup = self.client.lookup_token()

        accessor = lookup["data"]["accessor"]
        expire_time = lookup["data"]["expire_time"]

        return 'token accessor={}, expire_time={}, sleep_time={}'.format(accessor, expire_time, self.sleep_time)

    def renew(self):
        return self.client.renew_token()

    def run(self):
        while True:
            self.renew()

            print(self.info())

            if not self.forever:
                break

            time.sleep(self.sleep_time)



import base64
import json
import logging
import os

from functools import lru_cache

import vaulthelpers

from .settings import VAULT_SECRETS_ROOT, VAULT_WRITE_SECRETS_FILES
from .uri import SecretURI


class VaultEnvironment:
    """
    Traverses the OS environment looking for values that should be fetched from Vault
    """
    def __init__(self,
                 secrets_root: str = VAULT_SECRETS_ROOT,
                 write_secrets_files: bool = VAULT_WRITE_SECRETS_FILES):
        self.write_secrets_files = write_secrets_files
        self.secrets_root = secrets_root

    def _check_item(self, key: str, value: str):
        """
        Processes the environment variable at the name `key` with the given value

        look for environment values such as secret://vault/secret/something
        """
        uri = SecretURI(value)
        if not uri.is_secret:
            return

        secret = self.get_secret(uri)

        self.write_secret_file(key, secret)
        self.update_env(uri, key, secret)

    @property
    @lru_cache()
    def client(self):
        return vaulthelpers.client()

    def get_secret(self, uri: SecretURI):
        """
        Returns the secret found in Vault for the given secret URI
        """
        vault_response = self.client.secrets.kv.v2.read_secret_version(path=uri.path)

        return vault_response['data']['data'][uri.key]

    @property
    def logger(self):
        return logging.getLogger('{}.{}'.format(__name__, self.__class__.__name__))

    def run(self):
        if not self.client:
            # the vaulthelpers client function will warn already
            # self.logger.warning('Vault not configured; cannot fetch secrets')

            return

        for k, v in os.environ.items():
            self._check_item(k, v)

    @staticmethod
    def get_encoded(uri: SecretURI, value: str) -> str:
        """
        Returns the value encoded per the uri settings

        Args:
        uri: the variable URI
        value: the value to encode

        Returns:
            the encoded value
        """
        encoded = value

        encode = uri.encode
        if encode == 'base64':
            encoded = base64.b64encode(value.encode('utf8')).decode('utf8')
        elif encode == 'json':
            encoded = json.dumps(value)
        elif encode is not None:
            raise ValueError('Unknown encoding: {}'.format(encode))

        return '{}://{}'.format(encode, encoded) if encode else encoded

    def update_env(self, uri: SecretURI, key: str, value: str) -> None:
        """
        Python cannot update the parent environment, so what this does is print it to stdout

        The parent process is responsible for taking stdout and updating the runtime environment
        """
        # if the individual secret has opeted to not update the environment, skip printing
        if not uri.write_env:
            return

        encoded = self.get_encoded(uri, value)

        print('{}={}'.format(key, encoded))

    def write_secret_file(self, key: str, value: str) -> None:
        """
        Write the secret to a file in the secrets root directory
        """
        if not self.write_secrets_files:
            return

        # make the directory here instead of in a constructor so that an empty directory isn't left
        # behind if nothing is found
        if not os.path.exists(self.secrets_root):
            os.makedirs(self.secrets_root)

        full_path = os.path.join(self.secrets_root, key)
        with open(full_path, 'w') as fh:
            fh.write(value)



from unittest import TestCase

from vault_tools.environment import VaultEnvironment
from vault_tools.uri import SecretURI


class VaultEnvironmentTestCase(TestCase):
    def test_base64_encoding(self):
        uri = SecretURI('secret://vault/path?encode=base64')
        vault_environment = VaultEnvironment()

        encoded = vault_environment.get_encoded(uri, 'test')

        self.assertEquals('base64://dGVzdA==', encoded)

    def test_json_encoding(self):
        uri = SecretURI('secret://vault/path?encode=json')
        vault_environment = VaultEnvironment()

        encoded = vault_environment.get_encoded(uri, 'test')

        self.assertEquals('json://"test"', encoded)

    def test_no_encoding(self):
        uri = SecretURI('secret://vault/path')
        vault_environment = VaultEnvironment()

        encoded = vault_environment.get_encoded(uri, 'test')

        self.assertEquals('test', encoded)



#!/usr/bin/env python3
"""
Fetches secrets from vault from environment variables prefixed with secret://vault/[...]
"""
import urllib.parse as url_parser

from functools import lru_cache
from typing import Any

from conversion import convert_bool

SECRET_PREFIX = 'secret'


class SecretURI:
    """
    An interface around urllib functions to expose URI details as attributes

    ParseResult(scheme='secret', netloc='vault', path='/secret/something', params='', query='key=name', fragment='')
    """
    def __init__(self, uri: str, default_key: str = 'value', default_write_env: bool = True):
        self.default_key = default_key
        self.uri_s = uri  # the string version of the URI
        self.default_write_env = default_write_env

    @property
    def encode(self) -> str:
        return self.get_param('encode')

    def get_param(self, key: str, default: Any = None) -> str:
        """
        Returns the first item in a value list within the dictionary key
        """
        value = self.params.get(key)

        item = value
        if isinstance(item, list):
            item = item[0]

        return item or default

    @property
    def is_secret(self) -> bool:
        """
        Returns whether the scheme is secret://
        """
        return self.uri.scheme == SECRET_PREFIX

    @property
    def key(self) -> str:
        """
        Returns the key in the query params or the default key name
        """
        return self.get_param('key', default=self.default_key)

    @property
    @lru_cache()
    def params(self) -> dict:
        """
        Returns parsed query parameters
        """
        return url_parser.parse_qs(self.uri.query)

    @property
    @lru_cache()
    def uri(self) -> url_parser.ParseResult:
        """
        Returns parsed URI
        """
        return url_parser.urlparse(self.uri_s)

    @property
    @lru_cache()
    def path(self):
        """
        Returns the secret's path from the URI
        """
        return self.uri.path

    @property
    @lru_cache()
    def write_env(self) -> bool:
        write_env = self.get_param('write_env')

        return convert_bool(write_env) if write_env else self.default_write_env



import os

from conversion import convert_bool

# used for the token refresh command
# TODO: switch to vaulthelper for this?
VAULT_ADDR = os.environ.get("VAULT_ADDR")
VAULT_TOKEN = os.environ.get("VAULT_TOKEN")

VAULT_SECRETS_ROOT = os.environ.get('VAULT_SECRETS_ROOT', '/var/run/vault-secrets')
VAULT_WRITE_SECRETS_FILES = convert_bool(os.environ.get('VAULT_WRITE_SECRETS_FILES', 'true'))




#!/usr/bin/env python
import configparser

from setuptools import find_packages, setup

SRC_PREFIX = 'src'

packages = find_packages(SRC_PREFIX)


def get_required_packages():
    """
    Returns the packages used for install_requires

    This used to pin down every package in Pipfile.lock to the version, but that, in turn, broke
    downstream projects because it was way too strict.

    Now, this simply grabs all the items listed in the `Pipfile` `[packages]` section without version
    pinning
    """
    config = configparser.ConfigParser(strict=False)
    config.read('Pipfile')

    install_requires = sorted([x for x in config['packages']])

    return install_requires


def readme():
    with open('README.md') as f:
        return f.read()


if __name__ == '__main__':
    setup(
        name='vault-tools',
        url='https://github.com/openslate/vault-tools',
        author='OpenSlate',
        author_email='code@openslate.com',
        version='0.1.1',
        description='misc tools to work with vault',
        long_description=readme(),
        long_description_content_type='text/markdown',
        package_dir={'': 'src'},
        packages=packages,
        entry_points={
            'console_scripts': [
                'process-vault-environment = vault_tools.entrypoints:process_vault_environment',
                'renew-token = vault_tools.entrypoints:renew_token',
            ],
        },
        scripts=[
            'scripts/write-vault-environment',
        ],
        install_requires=get_required_packages()
    )



import argparse
import logging
import sys

from .environment import VaultEnvironment
from .refresh import VaultTokenRefresher
from .settings import VAULT_ADDR, VAULT_TOKEN

logging.basicConfig()


def process_vault_environment():
    """
    Traverses environment looking for entries to be retrieved from Vault
    """
    sys.exit(VaultEnvironment().run())


def renew_token():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--forever", action="store_true", help="run forever; default false"
    )
    parser.add_argument(
        "--sleep-time",
        type=int,
        default=3600,
        help="how long to sleep between renewals",
    )
    parser.add_argument(
        "--vault-addr",
        default=VAULT_ADDR,
        help="the vault address; defaults to the VAULT_ADDR environment var",
    )
    parser.add_argument(
        "token",
        default=VAULT_TOKEN,
        help="the vault token to renew; defaults to the VAULT_TOKEN environment var",
    )

    args = parser.parse_args()

    refresher = VaultTokenRefresher(
        args.vault_addr, args.token, sleep_time=args.sleep_time, forever=args.forever
    )

    refresher.run()



from unittest import TestCase

from vault_tools.uri import SecretURI


class VaultEnvironmentTestCase(TestCase):
    def test_write_env(self):
        uri = SecretURI('secret://vault/path?encode=base64')

        self.assertEquals(True, uri.write_env)

    def test_write_env_with_false_param(self):
        uri = SecretURI('secret://vault/path?encode=base64&write_env=false')

        self.assertEquals(False, uri.write_env)

    def test_write_env_with_true_param(self):
        uri = SecretURI('secret://vault/path?encode=base64&write_env=true')

        self.assertEquals(True, uri.write_env)
